
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <magick/api.h>
#include <magick/magick.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <xmagick.h>

static void add_pixel(unsigned long pixel, unsigned long *pixels,
		int *npixels, int max_pixels)
{
	int i;

	if (*npixels >= max_pixels) return;
	for (i = 0; i < *npixels; i++) {
		if (pixel == pixels[i]) return;
	}
	pixels[i++] = pixel;
	*npixels = i;
}

static int npixels_in_ximage(XImage *img, int max_pixels)
{
	int x, y;
	int npixels = 0;
	unsigned long *pixels = malloc(max_pixels*sizeof(*pixels));
	for (y = 0; y < img->height; y++) {
		for (x = 0; x < img->width; x++) {
			unsigned long pixel = XGetPixel(img, x, y);
			add_pixel(pixel, pixels, &npixels, max_pixels);
		}
	}
	return npixels;
}

int main(int argc, char **argv)
{
	Image *image, *image2;
	ImageInfo image_info;
	XImage *ximage;
	Display *display;
	int i;

#if MagickLibVersion >= 0x0520
	ExceptionInfo exception;
#if MagickLibVersion >= 0x0550
	InitializeMagick(*argv);
	GetExceptionInfo(&exception);
#endif
#endif

	if (getenv("XMagickOldAlloc")) {
		fprintf(stderr, "Using old alloc_color\n");
		XMagickOldAlloc = 1;
	}

	display = XOpenDisplay(NULL);
	if (display == NULL) {
		fprintf(stderr, "Can't open X display\n");
		exit(1);
	}

	if (argc != 3) {
		printf("Usage: %s infile outfile\n", argv[0]);
		return 0;
	}
for (i = 0; i < 10; i++) {
	GetImageInfo(&image_info);
	image_info.dither = 0;
	strcpy(image_info.filename, argv[1]);
#if MagickLibVersion < 0x0520
	image = ReadImage(&image_info);
#else	/* IM 5 */
	image = ReadImage(&image_info, &exception);
#endif
	if (image == NULL) return 1;

#if MagickLibVersion >= 0x0550
	printf("%ld colors in image\n", (long)GetNumberColors(image, NULL, &exception));
#else
	printf("%ld colors in image\n", (long)GetNumberColors(image, NULL));
#endif
	ximage = XMagickImageToXImage(display, image);
	printf("%d pixels in ximage\n", npixels_in_ximage(ximage, 256));
	image2 = XMagickXImageToImage(display, ximage);
#if MagickLibVersion >= 0x0550
	printf("%ld colors in image2\n", (long)GetNumberColors(image2, NULL, &exception));
#else
	printf("%ld colors in image2\n", (long)GetNumberColors(image2, NULL));
#endif
	strcpy(image2->filename, argv[2]);

	/* Save to disk */
	WriteImage(&image_info, image2);

	/* Free resources */
	DestroyImage(image);
	DestroyImage(image2);
	XDestroyImage(ximage);
}
	XCloseDisplay(display);

#if MagickLibVersion >= 0x0550
	DestroyExceptionInfo( &exception );
	DestroyMagick();
#endif

	return 0;
}
