/* 

Copyright (c) 1999-2004  Ulric Eriksson <ulric@siag.nu>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#include <stdio.h>
#include <stdlib.h>
#include <magick/api.h>
#include <magick/magick.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <limits.h>
#include <X11/Xatom.h>
#include "XCC.h"

#ifndef USHRT_MAX
#define USHRT_MAX 65535
#endif

/* This variable is no longer used, but must still exist
   to permit linking old programs */
int XMagickOldAlloc;

static XCC xcc = NULL;

static void die(char *p)
{
	fprintf(stderr, "%s\n", p);
	exit(EXIT_FAILURE);
}

static void init_if_needed(Display *dpy, Visual *vis, int use_private,
			int use_std, Atom std_cmap, Colormap *cmap)
{
	if (xcc) return;
	xcc = XmagickXCCCreate(dpy, vis, use_private, use_std, std_cmap, cmap);
}

static void alloc_color3(XCC xcc, XColor *color)
{
	color->pixel = XmagickXCCGetPixel(xcc, color->red, color->green, color->blue);
}

static void query_color3(XCC xcc, XColor *color)
{
	XmagickXCCQueryColor(xcc, color);
}

/* Convert Image to XImage */
XImage *XMagickImageToXImage(Display *display, Image *image)
{
	XImage *ximage = NULL;
	int x, y;
	int screen = DefaultScreen(display);
	Visual *visual = DefaultVisual(display, screen);
	int format = ZPixmap;
	int offset = 0;
	char *data = NULL;
	Colormap colormap = DefaultColormap(display, screen);
	int bitmap_pad = XBitmapPad(display);
	int bytes_per_line = 0;
	unsigned int width = image->columns;
	unsigned int height = image->rows;
	unsigned int depth = DefaultDepth(display, screen);
	XColor color;
	unsigned long pixel;
	int i;
#if MagickLibVersion < 0x0520
	float *red, *green, *blue, *opacity;
#else
	PixelPacket *pp;
#endif

#if MagickLibVersion >= 0x0550
printf("Isn't this pointless now?\n");
	/* RDB: Added Wed Dec 31 12:43:11 PST 2003 */
	ExceptionInfo	exception;
	GetExceptionInfo( &exception );

	if (exception.severity != UndefinedException)
				CatchException(&exception);
#endif

	init_if_needed(display, visual, 0, 1, XA_RGB_DEFAULT_MAP, &colormap);

#if MagickLibVersion < 0x0520

	red = malloc((width*height)*(sizeof *red));
	green = malloc((width*height)*(sizeof *green));
	blue = malloc((width*height)*(sizeof *blue));
	opacity = malloc((width*height)*(sizeof *opacity));

	if (red && green && blue && opacity) {
		GetPixels(image, red, green, blue, opacity);
		ximage = XCreateImage(display, visual, depth,
				format, offset, data,
				width, height, bitmap_pad, bytes_per_line);
		if (ximage == NULL) die("Can't create XImage");
		ximage->data = malloc(ximage->bytes_per_line*ximage->height);
		if (ximage->data == NULL) die("Can't allocate XImage data");
		for (y = 0; y < height; y++) {
			for (x = 0; x < width; x++) {
				i = x+y*width;
				color.red = USHRT_MAX*red[i];
				color.green = USHRT_MAX*green[i];
				color.blue = USHRT_MAX*blue[i];
				alloc_color3(xcc, &color);
				pixel = color.pixel;
				XPutPixel(ximage, x, y, pixel);
			}
		}
	}

	free(red);
	free(green);
	free(blue);
	free(opacity);

#else	/* ImageMagick 5 */

	pp = GetImagePixels(image, 0, 0, width, height);
	ximage = XCreateImage(display, visual, depth,
			format, offset, data,
			width, height, bitmap_pad, bytes_per_line);
	if (ximage == NULL) die("Can't create XImage\n");
	ximage->data = malloc(ximage->bytes_per_line*ximage->height);
	if (ximage->data == NULL) die("Can't allocate XImage data\n");
	for (y = 0; y < height; y++) {
		for (x = 0; x < width; x++) {
			i = x+y*width;
			color.red = USHRT_MAX*pp[i].red/MaxRGB;
			color.green = USHRT_MAX*pp[i].green/MaxRGB;
			color.blue = USHRT_MAX*pp[i].blue/MaxRGB;
			alloc_color3(xcc, &color);
			pixel = color.pixel;
			XPutPixel(ximage, x, y, pixel);
		}
	}
#endif
	return ximage;
}

/* Convert XImage to Image */
Image *XMagickXImageToImage(Display *display, XImage *ximage)
{
	Image *image = NULL;
	XColor color;
	unsigned long pixel;
	int i;
	int x, y;
	int screen = DefaultScreen(display);
	Colormap colormap = DefaultColormap(display, screen);
	unsigned int width = ximage->width;
	unsigned int height = ximage->height;
#if MagickLibVersion >= 0x0520
	PixelPacket *pp;
#endif

	init_if_needed(display, DefaultVisual(display, screen),
		0, 1, XA_RGB_DEFAULT_MAP, &colormap);

#if MagickLibVersion < 0x0520
	red = malloc((width*height)*(sizeof *red));
	green = malloc((width*height)*(sizeof *green));
	blue = malloc((width*height)*(sizeof *blue));
	opacity = malloc((width*height)*(sizeof *opacity));

	if (red && green && blue && opacity) {
		for (y = 0; y < height; y++) {
			for (x = 0; x < width; x++) {
				i = x+y*width;
				pixel = XGetPixel(ximage, x, y);
				color.pixel = pixel;
				query_color3(xcc, &color);
				red[i] = (float)color.red/USHRT_MAX;
				green[i] = (float)color.green/USHRT_MAX;
				blue[i] = (float)color.blue/USHRT_MAX;
				opacity[i] = 1.0;
			}
		}
		image = CreateImage(width, height, red, green, blue, opacity);
	}

	free(red);
	free(green);
	free(blue);
	free(opacity);
#else	/* ImageMagick 5 */
	image = AllocateImage(NULL);
	image->columns = width;
	image->rows = height;
	i = 0;
	for (y = 0; y < height; y++) {
		pp = SetImagePixels(image, 0, y, width, 1);
		for (x = 0; x < width; x++) {
			pixel = XGetPixel(ximage, x, y);
			color.pixel = pixel;
			query_color3(xcc, &color);
			pp[x].red = MaxRGB*color.red/USHRT_MAX;
			pp[x].green = MaxRGB*color.green/USHRT_MAX;
			pp[x].blue = MaxRGB*color.blue/USHRT_MAX;
			pp[x].opacity = OpaqueOpacity;
		}
		SyncImagePixels(image);
	}
#endif
	return image;
}

/* Set the number of bits to use when calculating the rgb value for
   alloc_color. Valid values are 1 to 8. Returns the new value.
*/
int XMagickRGBBits(int bits)
{
	fprintf(stderr, "The XMagickRGBBits function no longer does anything useful\n");
	return bits;
}

